@@ -1,6 +1,6 @@
 /* gpgmetools.h - Additional gpgme support functions for GPA.
    Copyright (C) 2002 Miguel Coca.
-   Copyright (C) 2005, 2008, 2009, 2012 g10 Code GmbH.
+   Copyright (C) 2005, 2008, 2009, 2012, 2014 g10 Code GmbH.
 
    This file is part of GPA
 
@@ -81,8 +81,10 @@ _gpa_gpgme_error (gpg_error_t err, const char *file, int line)
 }
 
 
+/* (Please use the gpa_gpgme_warning macros).  */
 void
-gpa_gpgme_warning_ext (gpg_error_t err, const char *desc)
+_gpa_gpgme_warning (gpg_error_t err, const char *desc,
+                    const char *file, int line)
 {
   char *argbuf = NULL;
   const char *arg;
@@ -100,21 +102,16 @@ gpa_gpgme_warning_ext (gpg_error_t err, const char *desc)
 
   message = g_strdup_printf
     (_("The GPGME library returned an unexpected\n"
-       "error. The error was:\n\n"
+       "error at %s:%d. The error was:\n\n"
        "\t%s\n\n"
-       "This is probably a bug in GPA.\n"
-       "GPA will now try to recover from this error."),
-     arg);
+       "This is either an installation problem or a bug in %s.\n"
+       "%s will now try to recover from this error."),
+     strip_path (file), line, arg, GPA_NAME, GPA_NAME);
   g_free (argbuf);
   gpa_window_error (message, NULL);
   g_free (message);
 }
 
-void
-gpa_gpgme_warning (gpg_error_t err)
-{
-  gpa_gpgme_warning_ext (err, NULL);
-}
 
 /* Initialize a gpgme_ctx_t for use with GPA.  */
 gpgme_ctx_t
@@ -138,7 +135,8 @@ gpa_gpgme_new (void)
 /* Write the contents of the gpgme_data_t object to the file.
    Receives a filehandle instead of the filename, so that the caller
    can make sure the file is accesible before putting anything into
-   data.  */
+   data.  This is only used for a TMP file, thus it is okay to
+   terminate the application on error. */
 void
 dump_data_to_file (gpgme_data_t data, FILE *file)
 {
@@ -293,37 +291,12 @@ gpa_gpgme_data_new_from_file (gpgme_data_t *data,
 }
 
 
-#ifdef G_OS_WIN32
-/* Convert newlines.  */
-void
-dos_to_unix (gchar *str, gsize *len)
-{
-  /* On Windows 2000, we need to convert \r\n to \n in the output for
-     cut & paste to work properly (otherwise, extra newlines will be
-     inserted).  */
-  gchar *src;
-  gchar *dst;
-
-  src = str;
-  dst = str;
-  while (*src)
-    {
-      if (src[0] == '\r' && src[1] == '\n')
-	src++;
-      *(dst++) = *(src++);
-    }
-  *dst = '\0';
-  *len = dst - str;
-}
-#endif
-
-
 /* Write the contents of the gpgme_data_t into the clipboard.  Assumes
-   that the data is ASCII.  */
-void
+   that the data is ASCII.  Return 0 on success.  */
+int
 dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
 {
-  char buffer[128];
+  char buffer[512];
   int nread;
   gchar *text = NULL;
   size_t len = 0;
@@ -332,27 +305,23 @@ dump_data_to_clipboard (gpgme_data_t data, GtkClipboard *clipboard)
   if (nread == -1)
     {
       gpa_window_error (strerror (errno), NULL);
-      exit (EXIT_FAILURE);
+      return -1;
     }
   while ((nread = gpgme_data_read (data, buffer, sizeof (buffer))) > 0)
     {
-      text = g_realloc (text, len + nread);
+      text = g_realloc (text, len + nread + 1);
       strncpy (text + len, buffer, nread);
       len += nread;
     }
   if (nread == -1)
     {
       gpa_window_error (strerror (errno), NULL);
-      exit (EXIT_FAILURE);
+      return -1;
     }
 
-#ifdef G_OS_WIN32
-  dos_to_unix (text, &len);
-#endif
-
   gtk_clipboard_set_text (clipboard, text, (int)len);
   g_free (text);
-  return;
+  return 0;
 }
 
 
@@ -553,44 +522,53 @@ get_gpg_connect_agent_path (void)
 
 
 /* Backup a key. It exports both the public and secret keys to a file.
-   Returns TRUE on success and FALSE on error. It displays errors to
-   the user.  */
+   IS_X509 tells the function that the fingerprint is from an X.509
+   key.  Returns TRUE on success and FALSE on error. It displays
+   errors to the user.  */
 gboolean
-gpa_backup_key (const gchar *fpr, const char *filename)
+gpa_backup_key (const gchar *fpr, const char *filename, int is_x509)
 {
-  gchar *header, *pub_key, *sec_key;
-  gchar *err;
-  FILE *file;
-  gint ret_code;
-  gchar *header_argv[] =
+  const char *header_argv[] =
     {
-      NULL, "--batch", "--no-tty", "--fingerprint", (gchar*) fpr, NULL
+      "", "--batch", "--no-tty", "--fingerprint",
+      (char*) fpr, NULL
     };
-  gchar *pub_argv[] =
+  const char *pub_argv[] =
     {
-      NULL, "--batch", "--no-tty", "--armor", "--export", (gchar*) fpr, NULL
+      "", "--batch", "--no-tty", "--armor", "--export",
+      (char*) fpr, NULL
     };
-  gchar *sec_argv[] =
+  const char *sec_argv[] =
     {
-      NULL, "--batch", "--no-tty", "--armor", "--export-secret-key",
-      (gchar*) fpr, NULL
+      "", "--batch", "--no-tty", "--armor", "--export-secret-key",
+      (char*) fpr, NULL
     };
-  const gchar *path;
-  mode_t mask;
+  const char *seccms_argv[] =
+    {
+      "", "--batch", "--no-tty", "--armor", "--export-secret-key-p12",
+      (char*) fpr, NULL
+    };
+  gpg_error_t err;
+  FILE *fp;
+  gpgme_data_t dfp = NULL;
+  const char *pgm;
+  gpgme_ctx_t ctx = NULL;
+  int result = FALSE;
 
   /* Get the gpg path.  */
-  path = get_gpg_path ();
-  g_return_val_if_fail (path && *path, FALSE);
+  if (is_x509)
+    pgm = get_gpgsm_path ();
+  else
+    pgm = get_gpg_path ();
+  g_return_val_if_fail (pgm && *pgm, FALSE);
 
-  /* Add the executable to the arg arrays */
-  header_argv[0] = (gchar*) path;
-  pub_argv[0] = (gchar*) path;
-  sec_argv[0] = (gchar*) path;
   /* Open the file */
-  mask = umask (0077);
-  file = g_fopen (filename, "w");
-  umask (mask);
-  if (!file)
+  {
+    mode_t mask = umask (0077);
+    fp = g_fopen (filename, "w");
+    umask (mask);
+  }
+  if (!fp)
     {
       gchar message[256];
       g_snprintf (message, sizeof(message), "%s: %s",
@@ -598,44 +576,62 @@ gpa_backup_key (const gchar *fpr, const char *filename)
       gpa_window_error (message, NULL);
       return FALSE;
     }
-  /* Get the keys and write them into the file */
-  fputs (_("************************************************************************\n"
-	   "* WARNING: This file is a backup of your secret key. Please keep it in *\n"
-	   "* a safe place.                                                        *\n"
-	   "************************************************************************\n\n"),
-	 file);
-  fputs (_("The key backed up in this file is:\n\n"), file);
-  if( !g_spawn_sync (NULL, header_argv, NULL, 0, NULL, NULL, &header,
-		     &err, &ret_code, NULL))
-    {
-      return FALSE;
-    }
-  fputs (header, file);
-  g_free (err);
-  g_free (header);
-  fputs ("\n", file);
-  if( !g_spawn_sync (NULL, pub_argv, NULL, 0, NULL, NULL, &pub_key,
-		     &err, &ret_code, NULL))
-    {
-      fclose (file);
-      return FALSE;
-    }
-  fputs (pub_key, file);
-  g_free (err);
-  g_free (pub_key);
-  fputs ("\n", file);
-  if( !g_spawn_sync (NULL, sec_argv, NULL, 0, NULL, NULL, &sec_key,
-		     &err, &ret_code, NULL))
-    {
-      fclose (file);
-      return FALSE;
-    }
-  fputs (sec_key, file);
-  g_free (err);
-  g_free (sec_key);
 
-  fclose (file);
-  return TRUE;
+  fputs (_(
+    "************************************************************************\n"
+    "* WARNING: This file is a backup of your secret key. Please keep it in *\n"
+    "* a safe place.                                                        *\n"
+    "************************************************************************\n"
+    "\n"), fp);
+
+  fputs (_("The key backed up in this file is:\n\n"), fp);
+  fflush (fp);
+
+  err = gpgme_data_new_from_stream (&dfp, fp);
+  if (err)
+    {
+      g_message ("error creating data object '%s': %s",
+                 filename, gpg_strerror (err));
+      goto leave;
+    }
+
+  ctx = gpa_gpgme_new ();
+  gpgme_set_protocol (ctx, GPGME_PROTOCOL_SPAWN);
+
+  err = gpgme_op_spawn (ctx, pgm, header_argv, NULL, dfp, NULL,
+                        GPGME_SPAWN_DETACHED|GPGME_SPAWN_ALLOW_SET_FG);
+  if (err)
+    {
+      g_message ("error running '%s' (1): %s", pgm, gpg_strerror (err));
+      goto leave;
+    }
+  gpgme_data_write (dfp, "\n", 1);
+
+  err = gpgme_op_spawn (ctx, pgm, pub_argv, NULL, dfp, NULL,
+                        GPGME_SPAWN_DETACHED|GPGME_SPAWN_ALLOW_SET_FG);
+  if (err)
+    {
+      g_message ("error running '%s' (2): %s", pgm, gpg_strerror (err));
+      goto leave;
+    }
+  gpgme_data_write (dfp, "\n", 1);
+
+  err = gpgme_op_spawn (ctx, pgm, is_x509? seccms_argv : sec_argv,
+                        NULL, dfp, NULL,
+                        GPGME_SPAWN_DETACHED|GPGME_SPAWN_ALLOW_SET_FG);
+  if (err)
+    {
+      g_message ("error running '%s' (3): %s", pgm, gpg_strerror (err));
+      goto leave;
+    }
+
+  result = TRUE;
+
+ leave:
+  gpgme_release (ctx);
+  gpgme_data_release (dfp);
+  fclose (fp);
+  return result;
 }
 
 
@@ -1352,71 +1348,8 @@ compare_version_strings (const char *my_version,
   return 0;
 }
 
+
 /* Try switching to the gpg2 backend or the one given by filename.  */
-void
-gpa_switch_to_gpg2 (const char *gpg_binary, const char *gpgsm_binary)
-{
-  const char *oldname;
-  char *newname;
-  char *p;
-
-  if (gpgsm_binary)
-    {
-      if (!access (gpgsm_binary, X_OK))
-        {
-          gpgme_set_engine_info (GPGME_PROTOCOL_CMS, gpgsm_binary, NULL);
-          g_message ("switched CMS engine to `%s'", gpgsm_binary);
-        }
-      else
-        g_message ("problem accessing CMS engine `%s'", gpgsm_binary);
-    }
-
-  if (gpg_binary)
-    {
-      if (!access (gpg_binary, X_OK))
-        {
-          gpgme_set_engine_info (GPGME_PROTOCOL_OpenPGP, gpg_binary, NULL);
-          g_message ("switched OpenPGP engine to `%s'", gpg_binary);
-          return;
-        }
-      g_message ("problem accessing OpenPGP engine `%s'", gpg_binary);
-    }
-
-  if (is_gpg_version_at_least ("2.0.0"))
-    return; /* Already using 2.0.  */
-
-  oldname = get_gpg_path ();
-  g_return_if_fail (oldname && *oldname);
-  newname = xmalloc (strlen (oldname) + 1 + 1);
-  strcpy (newname, oldname);
-#ifdef G_OS_WIN32
-# define OLD_NAME "gpg.exe"
-# define NEW_NAME "gpg2.exe"
-  for (p=newname; *p; p++)
-    if (*p == '\\')
-      *p = '/';
-#else
-# define OLD_NAME "gpg"
-# define NEW_NAME "gpg2"
-#endif /*G_OS_WIN32*/
-  p = strrchr (newname, '/');
-  if (p)
-    p++;
-  if (!strcmp (p, OLD_NAME))
-    {
-      strcpy (p, NEW_NAME);
-      if (!access (newname, X_OK))
-        {
-          gpgme_set_engine_info (GPGME_PROTOCOL_OpenPGP, newname, NULL);
-          g_message ("switched engine to `%s'", newname);
-        }
-    }
-#undef OLD_NAME
-#undef NEW_NAME
-  xfree (newname);
-}
-
-
 /* Return 1 if the gpg engine has at least version NEED_VERSION,
    otherwise 0.  */
 int
@@ -1435,20 +1368,21 @@ is_gpg_version_at_least (const char *need_version)
 }
 
 
-/* Structure used to communicate with gpg_simple_stderr_cb.  */
-struct gpg_simple_stderr_parm_s
+/* Structure used to communicate with gpg_simple_stdio_cb.  */
+struct gpg_simple_stdio_parm_s
 {
   gboolean (*cb)(void *opaque, char *line);
   void *cb_arg;
   GString *string;
+  int only_status_lines;
 };
 
 /* Helper for gpa_start_simple_gpg_command.  */
 static gboolean
-gpg_simple_stderr_cb (GIOChannel *channel, GIOCondition condition,
-                      void *user_data)
+gpg_simple_stdio_cb (GIOChannel *channel, GIOCondition condition,
+                     void *user_data)
 {
-  struct gpg_simple_stderr_parm_s *parm = user_data;
+  struct gpg_simple_stdio_parm_s *parm = user_data;
   GIOStatus status;
   char *line, *p;
 
@@ -1461,27 +1395,41 @@ gpg_simple_stderr_cb (GIOChannel *channel, GIOCondition condition,
         {
           line = parm->string->str;
 
+          /* Strip line terminator.  */
+          p = strchr (line, '\n');
+          if (p)
+            {
+              if (p > line && p[-1] == '\r')
+                p[-1] = 0;
+              else
+                *p = 0;
+            }
+
           /* We care only about status lines.  */
-          if (!strncmp (line, "[GNUPG:] ", 9))
+          if (parm->only_status_lines && !strncmp (line, "[GNUPG:] ", 9))
             {
               line += 9;
 
-              /* Strip line terminator.  */
-              p = strchr (line, '\n');
-              if (p)
-                {
-                  if (p > line && p[-1] == '\r')
-                    p[-1] = 0;
-                  else
-                    *p = 0;
-                }
-
               /* Call user callback.  */
               if (parm->cb && !parm->cb (parm->cb_arg, line))
                 {
                   /* User requested EOF.  */
                   goto cleanup;
                 }
+              /* Return direct so that we do not run into the G_IO_HUP
+                 check.  This is required to read all buffered input.  */
+              return TRUE;  /* Keep on watching this channel. */
+            }
+          else if (!parm->only_status_lines)
+            {
+              /* Call user callback.  */
+              if (parm->cb && !parm->cb (parm->cb_arg, line))
+                {
+                  /* User requested EOF.  */
+                  goto cleanup;
+                }
+
+              return TRUE;  /* Keep on watching this channel. */
             }
         }
       if (status != G_IO_STATUS_NORMAL && status != G_IO_STATUS_AGAIN )
@@ -1517,20 +1465,35 @@ gpg_simple_stderr_cb (GIOChannel *channel, GIOCondition condition,
    is used to call gpg-connect-agent.
 
    If the function returns success the provided callback CB is called
-   for each line received on STDERR.  EOF is send to this callback by
-   passing a LINE argument of NULL.  The callback may use this for
-   cleanup. If the callback returns FALSE, an EOF is forced with the
-   result that the callback is called once more with LINE set to NULL.  */
+   for each line received on stdout (respective stderr if USE_STADERR
+   is true).  EOF is send to this callback by passing a LINE as NULL.
+   The callback may use this for cleanup.  If the callback returns
+   FALSE, an EOF is forced so that the callback is called once more
+   with LINE set to NULL.
+
+   This function is used to run
+
+    gpgsm --learn-card
+    gpg-connect-agent NOP /bye
+
+   The problem is that under Windows g_spawn does not allow to specify
+   flags for the underlying CreateProcess.  Thus it is not possible to
+   create a detached process (i.e. without a console); the result is
+   that a console window pops up.  I can see two solutions: (1) Use a
+   wrapper process to start them detached, or (2) move the required
+   function into GPGME and use that new API.
+  */
 gpg_error_t
 gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
                               void *cb_arg, gpgme_protocol_t protocol,
+                              int use_stderr,
                               const char *first_arg, ...)
 {
   char *argv[24];
   int argc;
-  int fd_stderr;
+  int fd_stdio;
   GIOChannel *channel;
-  struct gpg_simple_stderr_parm_s *parm = NULL;
+  struct gpg_simple_stdio_parm_s *parm = NULL;
   char *freeme = NULL;
 
   if (protocol == GPGME_PROTOCOL_OpenPGP)
@@ -1543,7 +1506,14 @@ gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
     argv[0] = freeme = get_gpg_connect_agent_path ();
   else
     argv[0] = NULL;
-  g_return_val_if_fail (argv[0], gpg_error (GPG_ERR_INV_ARG));
+
+  if (!argv[0])
+    {
+      gpa_window_error (_("A required engine component is not installed."),
+                        NULL);
+      return gpg_error (GPG_ERR_INV_ARG);
+    }
+
   argc = 1;
   if (protocol != GPGME_PROTOCOL_GPGCONF
       && protocol != GPGME_PROTOCOL_ASSUAN)
@@ -1571,11 +1541,17 @@ gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
   parm->cb = cb;
   parm->cb_arg = cb_arg;
   parm->string = g_string_sized_new (200);
+  parm->only_status_lines = use_stderr;
 
   if (!g_spawn_async_with_pipes (NULL, argv, NULL,
-                                 (G_SPAWN_STDOUT_TO_DEV_NULL),
+                                 (use_stderr
+                                  ? G_SPAWN_STDOUT_TO_DEV_NULL
+                                  : G_SPAWN_STDERR_TO_DEV_NULL),
                                  NULL, NULL, NULL,
-                                 NULL, NULL, &fd_stderr, NULL))
+                                 NULL,
+                                 use_stderr? NULL : &fd_stdio,
+                                 use_stderr? &fd_stdio : NULL,
+                                 NULL))
     {
       gpa_window_error (_("Calling the crypto engine program failed."), NULL);
       xfree (parm);
@@ -1584,9 +1560,9 @@ gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
     }
   g_free (freeme);
 #ifdef G_OS_WIN32
-  channel = g_io_channel_win32_new_fd (fd_stderr);
+  channel = g_io_channel_win32_new_fd (fd_stdio);
 #else
-  channel = g_io_channel_unix_new (fd_stderr);
+  channel = g_io_channel_unix_new (fd_stdio);
 #endif
   g_io_channel_set_encoding (channel, NULL, NULL);
   /* Note that we need a buffered channel, so that we can use the read
@@ -1595,7 +1571,7 @@ gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
 
   /* Create a watch for the channel.  */
   if (!g_io_add_watch (channel, (G_IO_IN|G_IO_HUP),
-                       gpg_simple_stderr_cb, parm))
+                       gpg_simple_stdio_cb, parm))
     {
       g_debug ("error creating watch for gpg command");
       g_io_channel_unref (channel);
@@ -1613,8 +1589,29 @@ gpa_start_simple_gpg_command (gboolean (*cb)(void *opaque, char *line),
 void
 gpa_start_agent (void)
 {
-  gpa_start_simple_gpg_command (NULL, NULL, GPGME_PROTOCOL_ASSUAN,
-                                "NOP", "/bye", NULL);
+  gpg_error_t err;
+  gpgme_ctx_t ctx;
+  char *pgm;
+  const char *argv[3];
+
+  pgm = get_gpg_connect_agent_path ();
+  if (!pgm)
+    {
+      g_message ("tool to start the agent is not available");
+      return;
+    }
+
+  ctx = gpa_gpgme_new ();
+  gpgme_set_protocol (ctx, GPGME_PROTOCOL_SPAWN);
+  argv[0] = "";   /* Auto-insert the basename.  */
+  argv[1] = "NOP";
+  argv[2] = NULL;
+  err = gpgme_op_spawn (ctx, pgm, argv, NULL, NULL, NULL, GPGME_SPAWN_DETACHED);
+  if (err)
+    g_message ("error running '%s': %s", pgm, gpg_strerror (err));
+  g_free (pgm);
+  gpgme_release (ctx);
+
 }
 
 
